clear; clc; % close all;

% ----- Im-Bessel Parameters ----- %
xi = 0.70;
m = 3;
sigma = 1e-3;
delta = sqrt(sigma^2/8*(1-xi)/sqrt(xi));
% -------------------------------- %

% ----- Grids ----- %
N = 512;
del = sigma/55;
L = N*del;
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*del);
[phisrc,rhosrc] = cart2pol(xsrc,ysrc);
rho_u = linspace(min(rhosrc(:)),max(rhosrc(:)),N);
[fx,fy] = meshgrid((-N/2:N/2-1)*1/L);
[~,rhofx] = cart2pol(fx,fy);
rhofx_u = linspace(min(rhofx(:)),max(rhofx(:)),N);
[rho_u,rhofx_u] = meshgrid(rho_u,rhofx_u);
[x1,y2] = meshgrid((-N/2:N/2-1)*L/N);
[y1,x2] = meshgrid((-N/2:N/2-1)*L/N);
[x1a,x2a] = meshgrid((-N/2:N/2-1)*L/N);
[~,y1a] = meshgrid((-N/2:N/2-1)*L/N);
% ----------------- %

% ----- Im-Bessel Theory ----- %
Sthy = Im_Bessel_W(xsrc,ysrc,xsrc,ysrc,xi,m,sigma);
Wx1y2thy = Im_Bessel_W(x1,0,0,y2,xi,m,sigma);
Wy1x2thy = Im_Bessel_W(0,y1,x2,0,xi,m,sigma);
Wx1x2thy = Im_Bessel_W(x1a,delta,x2a,delta,xi,m,sigma);
Wx1y1thy = Im_Bessel_W(x1a,y1a,delta,delta,xi,m,sigma);
% ---------------------------- %

% ----- Im-Bessel Simulation ----- %
numscreens = 100e3;
S = zeros(N);
Wx1y2 = zeros(N);
Wy1x2 = zeros(N);
Wx1x2 = zeros(N);
Wx1y1 = zeros(N);
for nn = 1:numscreens
    disp(nn);
    r = 1/sqrt(2*pi)*repmat(randn(N,1)+1j*randn(N,1),[1 N]);
    Phi = exp(-1/2*(2*pi)^2*delta^2*rhofx_u.^2);    
    Trho = 1/sqrt(2)*2*pi*delta/sqrt(pi)*2*pi ...
        *trapz(sqrt(rhofx_u).*r.*Phi.*besselj(m,2*pi*rho_u.*rhofx_u),1) ...
        *sqrt(mean(diff(rhofx_u(:,1))));       
    T = interp1(rho_u(1,:),Trho,rhosrc,'linear',nan);
    U = sqrt(xi^(-m/2)/(1-xi))*exp(-1j*m*phisrc) ...
        .*exp(-(1-sqrt(xi))^2/(1-xi)*rhosrc.^2/sigma^2).*T;
    S = S + abs(U).^2;
    Scorr(nn) = corr2(S/nn,Sthy);
    Wx1y2 = Wx1y2 + transpose(U(N/2+1,:))*(U(:,N/2+1)');
    Wy1x2 = Wy1x2 + U(:,N/2+1)*conj(U(N/2+1,:));
    Ui = interp2(xsrc,ysrc,U,xsrc,delta*ones(N),'spline');
    Wx1x2 = Wx1x2 + transpose(Ui(N/2+1,:))*conj(Ui(N/2+1,:));
    Wx1y1 = Wx1y1 + U.*conj(interp2(xsrc,ysrc,U,delta,delta,'spline'));
end
S = S/numscreens;
Wx1y2 = transpose(Wx1y2/numscreens);
Wy1x2 = transpose(Wy1x2/numscreens);
Wx1x2 = transpose(Wx1x2/numscreens);
Wx1y1 = Wx1y1/numscreens;
% -------------------------------- %

save(['Im_Bessel_results_' num2str(numscreens) '.mat']);

figure; 
subplot(2,2,1);
imagesc(xsrc(1,:)*1000,ysrc(:,1)*1000,Sthy); axis image; colorbar;
subplot(2,2,2);
imagesc(xsrc(1,:)*1000,ysrc(:,1)*1000,S); axis image; colorbar;
subplot(2,2,3);
plot(xsrc(1,:)*1000,Sthy(N/2+1,:));
hold on;
plot(xsrc(1,:)*1000,S(N/2+1,:));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,2,4);
plot(ysrc(:,1)*1000,Sthy(:,N/2+1));
hold on;
plot(ysrc(:,1)*1000,S(:,N/2+1));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y2thy)); axis image; colorbar;

subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wy1x2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wy1x2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wy1x2thy)); axis image; colorbar;
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wy1x2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1x2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1x2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1x2thy)); axis image; colorbar;

subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1x2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y1thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:)*1000,y2(:,1)*1000,real(Wx1y1)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:)*1000,real(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,real(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1)*1000,real(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,real(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y1thy)); axis image; colorbar;
subplot(2,4,6);
imagesc(x1(1,:)*1000,y2(:,1)*1000,imag(Wx1y1)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:)*1000,imag(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:)*1000,imag(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1)*1000,imag(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1)*1000,imag(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

