clear; clc; % close all;

% ----- RHNUC Parameters ----- %
sigma_x = 1.5e-3;
sigma_y = 1e-3;
delta_x = 1.25e-3;
delta_y = 1.1e-3;
x0 = 0.5e-3;
y0 = -0.6e-3;
m = 2;
n = 1;
% ---------------------------- %

% ----- Grids ----- %
N = 512;
L = 10e-3;
[xsrc,ysrc] = meshgrid((-N/2:N/2-1)*L/N);
[~,rhosrc] = cart2pol(xsrc,ysrc);
L_xi_eta = (sqrt(2)*L/2 + max(abs([x0 y0])))^2;
[xi,eta] = meshgrid((-N/2:N/2-1)*2*L_xi_eta/N);
deltaf = 1/(2*L_xi_eta);
[fxi,feta] = meshgrid((-N/2:N/2-1)*deltaf);
[x1,y2] = meshgrid((-N/2:N/2-1)*L/N);
[y1,x2] = meshgrid((-N/2:N/2-1)*L/N);
[x1a,x2a] = meshgrid((-N/2:N/2-1)*L/N);
[~,y1a] = meshgrid((-N/2:N/2-1)*L/N);
% ----------------- %

% ----- RHNUC Theory ----- %
Sthy = RHNUC_W(sigma_x,sigma_y,delta_x,delta_y,m,n,x0,y0,xsrc,ysrc,xsrc,ysrc);
Wx1y2thy = RHNUC_W(sigma_x,sigma_y,delta_x,delta_y,m,n,x0,y0,x1,0,0,y2);
Wy1x2thy = RHNUC_W(sigma_x,sigma_y,delta_x,delta_y,m,n,x0,y0,0,y1,x2,0);
Wx1x2thy = RHNUC_W(sigma_x,sigma_y,delta_x,delta_y,m,n,x0,y0,x1a,0,x2a,0);
Wx1y1thy = RHNUC_W(sigma_x,sigma_y,delta_x,delta_y,m,n,x0,y0,x1a,y1a,0,0);
% ------------------------ %

% ----- RHNUC Simulation ----- %
Phi_sqrt = abs(fxi).^m.*abs(feta).^n ...
    .*exp(-pi^2*delta_x^4*fxi.^2/2).*exp(-pi^2*delta_y^4*feta.^2/2);

numscreens = 100e3;
S = zeros(N);
Wx1y2 = zeros(N);
Wy1x2 = zeros(N);
Wx1x2 = zeros(N);
Wx1y1 = zeros(N);
for nn = 1:numscreens
    disp(nn);
    r = randn(N) + 1j*randn(N);
    T = (delta_x/sqrt(2))^(2*m+1)*(delta_y/sqrt(2))^(2*n+1) ...
        /sqrt(2*gamma(m+1/2)*gamma(n+1/2))*(2*pi)^(m+1/2)*(2*pi)^(n+1/2) ...
        *fftshift(fft2(fftshift(r.*Phi_sqrt)))*sqrt(deltaf^2);
    T = T(N/2+1:end,N/2+1:end);
    T = interp2(sqrt(xi(N/2+1:end,N/2+1:end)),sqrt(eta(N/2+1:end,N/2+1:end)), ...
        T,abs(xsrc-x0),abs(ysrc-y0),'spline');
    U = exp(-xsrc.^2/(4*sigma_x^2)).*exp(-ysrc.^2/(4*sigma_y^2)).*T;
    S = S + abs(U).^2;
    Scorr(nn) = corr2(S/nn,Sthy);
    Wx1y2 = Wx1y2 + transpose(U(N/2+1,:))*(U(:,N/2+1)');
    Wy1x2 = Wy1x2 + U(:,N/2+1)*conj(U(N/2+1,:));    
    Wx1x2 = Wx1x2 + transpose(U(N/2+1,:))*conj(U(N/2+1,:));
    Wx1y1 = Wx1y1 + U.*conj(U(N/2+1,N/2+1));
end
S = S/numscreens;
Wx1y2 = transpose(Wx1y2/numscreens);
Wy1x2 = transpose(Wy1x2/numscreens);
Wx1x2 = transpose(Wx1x2/numscreens);
Wx1y1 = Wx1y1/numscreens;
% -------------------------------- %

save(['RHNUC_results_' num2str(numscreens) '.mat']);

figure; 
subplot(2,2,1);
imagesc(xsrc(1,:),ysrc(:,1),Sthy); axis image; colorbar;
subplot(2,2,2);
imagesc(xsrc(1,:),ysrc(:,1),S); axis image; colorbar;
subplot(2,2,3);
plot(xsrc(1,:),Sthy(N/2+1,:));
hold on;
plot(xsrc(1,:),S(N/2+1,:));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,2,4);
plot(ysrc(:,1),Sthy(:,N/2+1));
hold on;
plot(ysrc(:,1),S(:,N/2+1));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:),y2(:,1),real(Wx1y2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:),y2(:,1),real(Wx1y2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:),real(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:),real(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1),real(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1),real(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:),y2(:,1),imag(Wx1y2thy)); axis image; colorbar;
subplot(2,4,6);
imagesc(x1(1,:),y2(:,1),imag(Wx1y2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:),imag(Wx1y2thy(N/2+1,:)));
hold on;
plot(x1(1,:),imag(Wx1y2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1),imag(Wx1y2thy(:,N/2+1)));
hold on;
plot(y2(:,1),imag(Wx1y2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:),y2(:,1),real(Wy1x2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:),y2(:,1),real(Wy1x2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:),real(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:),real(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1),real(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1),real(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:),y2(:,1),imag(Wy1x2thy)); axis image; colorbar;

subplot(2,4,6);
imagesc(x1(1,:),y2(:,1),imag(Wy1x2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:),imag(Wy1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:),imag(Wy1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1),imag(Wy1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1),imag(Wy1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:),y2(:,1),real(Wx1x2thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:),y2(:,1),real(Wx1x2)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:),real(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:),real(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1),real(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1),real(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:),y2(:,1),imag(Wx1x2thy)); axis image; colorbar;
subplot(2,4,6);
imagesc(x1(1,:),y2(:,1),imag(Wx1x2)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:),imag(Wx1x2thy(N/2+1,:)));
hold on;
plot(x1(1,:),imag(Wx1x2(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1),imag(Wx1x2thy(:,N/2+1)));
hold on;
plot(y2(:,1),imag(Wx1x2(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');

figure;
subplot(2,4,1);
imagesc(x1(1,:),y2(:,1),real(Wx1y1thy)); axis image; colorbar;
subplot(2,4,2);
imagesc(x1(1,:),y2(:,1),real(Wx1y1)); axis image; colorbar;
subplot(2,4,3);
plot(x1(1,:),real(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:),real(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,4);
plot(y2(:,1),real(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1),real(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,5);
imagesc(x1(1,:),y2(:,1),imag(Wx1y1thy)); axis image; colorbar;
subplot(2,4,6);
imagesc(x1(1,:),y2(:,1),imag(Wx1y1)); axis image; colorbar;
subplot(2,4,7);
plot(x1(1,:),imag(Wx1y1thy(N/2+1,:)));
hold on;
plot(x1(1,:),imag(Wx1y1(N/2+1,:)));
axis tight; grid on;
legend('Thy','Sim');
subplot(2,4,8);
plot(y2(:,1),imag(Wx1y1thy(:,N/2+1)));
hold on;
plot(y2(:,1),imag(Wx1y1(:,N/2+1)));
axis tight; grid on;
legend('Thy','Sim');


